<?php

/**
 * @file
 * Contains \Drupal\uniplace_installer\Form\AdsenseMainSettings.
 */

namespace Drupal\uniplace_installer\Form;

use Drupal\Component\Utility\Html;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;

//use Drupal\adsense\AdsenseAdBase;

/**
 * Class UniplaceInstallerMainSettings.
 */
class UniplaceInstallerMainSettings extends ConfigFormBase {

  /**
   * @var PATH_TO_CODE string     путь к папке с кодом php-клиента uniplace
   * @var CONFIG_FILE_NAME string  имя файла конфига // uniplacer_config.php
   * @var DEF_CHARSET string      кодировка по умолчанию
   */
  const PATH_TO_CODE = 'codes/php/uniplace_id/';
  const CONFIG_FILE_NAME = 'uniplacer_config.php';
  const DEF_CHARSET = 'utf-8';
  
  /**
  * {@inheritdoc}
  */
  public function getFormId() {
    return 'uniplace_installer_main_settings';
  }

  /**
  * {@inheritdoc}
  */
  protected function getEditableConfigNames() {
    return ['uniplace_installer.settings'];
  }

  /**
  * {@inheritdoc}
  */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = \Drupal::config('uniplace_installer.settings');

    $form['help'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => t('Note'),
    ];

    $form['help']['help'] = ['#markup' => 'After setting up, please, add "Uniplace Link Block" in sidebar your active theme on "Structure" - "Block scheme".'];
    
    
    $form['uniplace_site_hash'] = [
      '#type' => 'textfield',
      '#title' => t('Site hash (in Uniplace system)'),
      '#required' => TRUE,
      '#default_value' => $config->get('uniplace_site_hash'),
      '#description' => t('This is the site hash in Uniplace system for the site owner. Get this in your Uniplace account. It should be similar to %id.', [
        '%id' => 'da20807428fb__________________',
      ]
      ),
    ];
    
    $uniplace_charset = $config->get('uniplace_charset');

    $form['uniplace_charset'] = [
      '#type' => 'textfield',
      '#title' => t('Anchor text encoding'),
      '#required' => TRUE,
      '#default_value' => ($uniplace_charset != '') ? $uniplace_charset : 'utf-8',
      '#description' => t('If the links are displayed in incorrect coding please change the encoding here. Default charset %charset.', [
        '%charset' => 'utf-8',
      ]
      ),
    ];

    $form['uniplace_installer_id_module'] = [
      '#type' => 'hidden',
      '#value' => 'uniplace_installer_basic',
    ];


    return parent::buildForm($form, $form_state);
  }

  /**
  * {@inheritdoc}
  */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    
//    echo '$form<pre>'; print_r($form); echo '</pre>';
//    echo '$form_state<pre>'; print_r($form_state); echo '</pre>';

    $config = \Drupal::configFactory()->getEditable('uniplace_installer.settings');
    $form_state->cleanValues();
    
    $values = $form_state->getValues();
    
    $values['uniplace_charset'] = ($values['uniplace_charset'] != '') ? $values['uniplace_charset'] : self::DEF_CHARSET;
    
    foreach ($values as $key => $value) {
      $config->set($key, Html::escape($value));
    }
    
    $config->save();
    
    if ($values['uniplace_site_hash'] != '') {
      $ret = self::code_files_install($values['uniplace_site_hash']);
    }
    
  }

  /**
   * инсталлирует файлы php-клиента системы uniplace с $uniplace_site_hash в корень сайта
   * создан для будущего расширения функциональности
   * 
   * @param  string $uniplace_site_hash хеш сайта в системе uniplace
   * @return bool
   */
  public static function code_files_install($uniplace_site_hash)
  {
    $ret = false;
    $type_mess = '';
    $txt = '';
    
    if ($uniplace_site_hash != '')
    {
      $ret = self::code_files_copy($uniplace_site_hash);
      if ($ret)
      {
        drupal_set_message(t('Code files installed'));    
      }
      else
      {
        drupal_set_message(t('Code files NOT installed'), 'error');    
      }
    }
    else
    {
      drupal_set_message(t('Uniplace site hash is empty. Please enter the Uniplace site hash.'), 'error');    
    }

    return $ret;
  }
  
  /**
   * создает папку с именем $uniplace_site_hash в корне сайта с правами 0755
   * копирует файлы php-клиента системы uniplace в папку с именем $uniplace_site_hash 
   * 
   * @param  string $uniplace_site_hash хеш сайта в системе uniplace
   * @return bool
   */
  public static function code_files_copy($uniplace_site_hash)
  {
    $uniplace_site_hash = trim($uniplace_site_hash);
    $ret = false;
    
    if ($uniplace_site_hash != '') {
      $ret = self::create_config_file($uniplace_site_hash);
      
      if ($ret) {
        $path = self::get_document_root() . '/' . $uniplace_site_hash;
        $path = self::prepare_path($path);  // windows
        
        if (!file_exists($path)) {
          $ret = mkdir($path, 0755);
          
          if (true === $ret) {
            $ret = chmod($path, 0755);
            
            if (!$ret) {
              $err = sprintf(t('Warning: Can\'t change folder permissions to 0755 and work may be incorrect. Please change folder permissions to 0755 %s'), $path);
              drupal_set_message($err, 'warning');    
            }
            
            $path_to_code = dirname(dirname((__DIR__))) . '/' . self::PATH_TO_CODE;
            $path_to_code = self::prepare_path($path_to_code);  // windows
          
            $files = scandir($path_to_code);
            
            if (!empty($files) && is_array($files)) {
              foreach ($files as $file) {
                if ($file != "." && $file != "..") {
                  copy("$path_to_code/$file", "$path/$file"); 
                }
              }
            }
            else {
              $err = sprintf(t('Directory with Uniplace client code not found: %s'), $path_to_code);
              drupal_set_message($error, 'error');
              $ret = false;
            }
          }
          else {
            drupal_set_message(t('Can\'t create directory with Uniplace client. Please check parent folder permissions'), 'error');    
            $ret = false;
          }
        }
        else {
          $err = sprintf(t('Warning: Directory with Uniplace client exists already: %s'), $path);
          drupal_set_message($err, 'warning');    
          $ret = false;
        }
      }
      else {
        $err = sprintf(t('Can\'t create config file - %s Please check permissions for folder or this file if this file exists already.'), $_SERVER['DOCUMENT_ROOT'] . '/' . self::CONFIG_FILE_NAME);
        drupal_set_message($err, 'error');    
      }
    }
    else {
      drupal_set_message(t('Uniplace site hash is empty. Please enter the Uniplace site hash.'), 'error');    
    }
    
    return $ret;
  }
  
  /**
   * заменяет в строке пути $path обратные слеши на прямые
   * для окружения на платфоме windows
   * 
   * @return string
   */
  public static function prepare_path($path)
  {
    return str_replace('\\', '/', $path);  // windows
  }
  
  /**
   * формирует и создает файл конфигурации self::CONFIG_FILE_NAME в корне сайта
   * 
   * @param  string $uniplace_site_hash хеш сайта в системе uniplace
   * @return bool
   */
  public static function create_config_file($uniplace_site_hash)
  {
    $uniplace_site_hash = trim($uniplace_site_hash);
    
    // @fixme: вытащить из настроек модуля mod_uniplace_installer
    // $params = JModuleHelper::getParams('mod_uniplace_installer');  //self::DEF_CHARSET
    // $uniplace_charset = $params->get('uniplace_charset');
    
    $config = \Drupal::config('uniplace_installer.settings');
    $uniplace_charset = $config->get('uniplace_charset');
    $uniplace_charset = ($uniplace_charset != '') ? $uniplace_charset : self::DEF_CHARSET;

    $ret = false;
    
    if ($uniplace_site_hash != '')
    {
      $content_config_file = "<?php" . PHP_EOL;
      $content_config_file .= "\tdefine('_UNIPLACE_USER_', '$uniplace_site_hash'); // Site hash" . PHP_EOL;
      $content_config_file .= "\tdefine('_UNIPLACE_CHARSET_', '$uniplace_charset'); // Anchor text encoding" . PHP_EOL;
      $content_config_file .= "?>";
      
      $config_file = self::get_document_root() . '/' . self::CONFIG_FILE_NAME;
      
      $ret = file_put_contents($config_file, $content_config_file);
      
      if ($ret)
      {
        $res_permissions = chmod($config_file, 0755);
        if (!$res_permissions)
        {
          $err = sprintf(t('Warning: Can\'t change config file permissions to 0755 and work may be incorrect. Please change config file permissions to 0755 %s'), $config_file);
          drupal_set_message($err, 'warning');    
        }
      }
    }
    else
    {
      drupal_set_message(t('Uniplace site hash is empty. Please enter the Uniplace site hash.'), 'error');    
    }
    
    return ((false === $ret) ? false : true);
  }

  /**
   * возвращает корректный DOCUMENT_ROOT
   * в некоторых системах может быть ненастроен
   * 
   * @return bool|string
   */
  public static function get_document_root()
  {
     return trim(realpath(getcwd()));
  }
  
}
